// $Id: Queue.java,v 1.1 2007/10/29 21:05:05 leavens Exp leavens $
/** A primitive concurrent Queue of Objects, used to demonstrate
 * concurrency features of Java, including synchronized methods, the
 * wait method, and the notifyAll method.
 * @author Gary T. Leavens
 */
public class Queue {
    private int SIZE = 10;
    /** the elements in the queue. */
    private Object[] elems = new Object[SIZE];
    /** the number of elements currently in the queue. */
    private int numElems = 0;
    /** where to put the next element */
    private int toPut = 0;
    /** where to get the next element */
    private int toGet = 0;

    /** Put the argument in the Queue, or wait if no space is available. */
    public synchronized void put(Object x) {
        while (numElems >= SIZE) {
            try {
                wait();
            } catch (InterruptedException e) {
                // ignore it
            }
        }
        //@ assert numElems < SIZE;
        elems[toPut] = x;
        toPut = (toPut+1) % SIZE;
        numElems++;
        notifyAll();
    }

    /** Get the next element in the Queue, or wait if none are available. */
    public synchronized Object get() {
        while (numElems == 0) {
            try {
                wait();
            } catch (InterruptedException e) {
                // ignore it
            }
        }
        //@ assert numElems > 0;
        Object ret = elems[toGet];
        toGet = (toGet+1) % SIZE;
        numElems--;
        notifyAll();
        return ret;
    }
}
